<?php
/*
 * Subscription statuses:
 * 0 / null = No subscription
 * 1 = Active subscription
 * 2 = Inactive / Invalid Payment Method / Expired
 * 3 = Cancelled
 */
class subController {
    private $dbc;
    var $gateway;
    
    function __construct($gateway) {
    	$this->dbc = new DBCon();
        $this->gateway = $gateway;
    }
    
    function show() {
        if (validate_subscriber()) {
            $status = get_current_user_meta("bt_sub_status");
            $id = get_current_user_meta("bt_sub_id");
            $end_date = get_current_user_meta("bt_sub_expiration");
            include_once(ABSPATH . 'wp-content/plugins/loa-braintree/views/subscription_status.php');
            $token = $clientToken = $this->gateway->clientToken()->generate();
            include_once(ABSPATH . 'wp-content/plugins/loa-braintree/views/subscription_update.php');
        } else if (validate_user()) {
            $token = $clientToken = $this->gateway->clientToken()->generate();
            include_once(ABSPATH . 'wp-content/plugins/loa-braintree/views/subscription_checkout.php');
        } else {
            wp_redirect(home_url());
            exit();
        }
    }

    function process() {
        $success = -1;
        $new = true;
        $cancelled = (get_current_user_meta("bt_sub_status") == 3) ? true : false;
        if (isset($_POST["payment_method_nonce"]) && (validate_user() || $cancelled)) {
            if ($cancelled) {
                $new = false;
            }
            $success = $this->create_subscription();
            include_once(ABSPATH . 'wp-content/plugins/loa-braintree/views/subscription_checkout_complete.php');
        } else if (isset($_POST["payment_method_nonce"]) && validate_subscriber()) {
            $success = $this->update_subscription();
            $new = false;
            include_once(ABSPATH . 'wp-content/plugins/loa-braintree/views/subscription_checkout_complete.php');
        } else {
            wp_redirect(home_url());
            exit();
        }
    }
    
    function create_subscription() {
        $success = false;
        $current_user = wp_get_current_user();
        $email = (string) $current_user->user_email;
        $result = $this->gateway->customer()->create([
            'firstName' => $_POST["first_name"],
            'lastName' => $_POST["last_name"],
            'email' => $email,
            'creditCard' => array(
                'paymentMethodNonce' => $_POST["payment_method_nonce"],
                'options' => array(
                    'verifyCard' => true
                )
            )
        ]);

        if ($result->success) {
            $status = $result->customer->creditCards[0]->verifications[0]["status"];
            if ($status != 'processor_declined' && $status != 'gateway_rejected' && $status != 'failed') {
                $resultSub = $this->gateway->subscription()->create([
                    'paymentMethodToken' => $result->customer->creditCards[0]->token,
                    'planId' => 'monthly-basic'
                ]);
                if ($resultSub->success == true && $resultSub->subscription->status == "Active") {
                    update_current_user_meta('bt_subscription', 1);
                    update_current_user_meta('bt_sub_id', $resultSub->subscription->id);
                    update_current_user_meta('bt_cust_id', $result->customer->id);
                    update_current_user_meta('bt_sub_status', 1);
                    update_current_user_meta('bt_sub_started', date('Y-m-d H:i:s'));
                    update_current_user_meta('bt_sub_expiration', date('Y-m-d H:i:s', strtotime("+1 month")));
                    update_current_user_meta('bt_sub_charge_day', date('d'));
                    update_current_user_meta('bt_sub_months', 0);
                    $this->sendUpdateToLogin();
                    $this->sendNewSubscriptionEmail($current_user);
                    $success = true;
                }
            }
        }
        
        return $success;
    }
    
    function update_subscription() {
        $success = false;
        $updateResult = $this->gateway->customer()->update(
            get_current_user_meta("bt_cust_id"),
            [
              'creditCard' => array(
                    'paymentMethodNonce' => $_POST["payment_method_nonce"],
                    'options' => array(
                        'verifyCard' => true
                    )
                )
            ]
        );

        if ($updateResult->success) {
            $status = $updateResult->customer->creditCards[0]->verifications[0]["status"];
            if ($status != 'processor_declined' && $status != 'gateway_rejected' && $status != 'failed') {
                 $result = $this->gateway->subscription()->update(get_current_user_meta("bt_sub_id"), [
                    'paymentMethodToken' => $updateResult->customer->creditCards[0]->token
                ]);
                if ($result->success == true && $result->subscription->status == "Active") {
                    update_current_user_meta('bt_sub_status', 1);
                    $this->sendUpdateToLogin();
                    $this->sendNewSubscriptionEmail($current_user);
                    $success = true;
                }
            }
        }
        
        return $success;
    }
    
    function cancel() {
        $success = false;
        if (validate_subscriber()) {
            $result = $this->gateway->subscription()->cancel(get_current_user_meta("bt_sub_id"));
            if (!empty($result->errors)) {
                $success = true;
                update_current_user_meta('bt_sub_status', 3);
                $this->sendUpdateToLogin();
            }
        }
    }
    
    function sendUpdateToLogin() {
        global $wpdb;

        $url = 'http://internal-loginserver-prod-internal-400560044.us-east-1.elb.amazonaws.com:8080/UpdateSubscription';
        $log = fopen('/var/www/html/wp-content/sublog.txt', 'a+');

        $so_user_id = get_current_user_meta("so_user_id");
        if($so_user_id)
        {
            $data = array('userId' => $so_user_id, 
                          'premiumSubscriptionExpire' => get_current_user_meta("bt_sub_expiration"), 
                          'premiumSubscriptionMonths' => get_current_user_meta("bt_sub_months"));

            // use key 'http' even if you send the request to https://...
            $options = array(
                'http' => array(
                    'header'  => "Content-type: application/json\r\n" .
                    "LOA_PUB_KEY: 5CfGvUpb5n9JSETWQ7ArSQ==.flfoFQ5F3MVnQGN/JXqhXER9H6M7pzloejB4ELUcusE=\r\n" .
                    "LOA_PRI_KEY: hX1NZt2rCU4TJvzWQFI6rDVqHZJbK//czdjDz8J3ZJ8OQVV4kRDa+0qKDMJioYMUIbN8NA4mYcGJxeuCLY/3AQ==\r\n",
                    'method'  => 'POST',
                    'content' => json_encode($data),
                    'timeout' => '45'
                ),
            );
            $context  = stream_context_create($options);
            $result = file_get_contents($url, false, $context);
            if ($result !== FALSE) {
                $wpdb->delete('wp_failed_subscription', array('so_user_id' => $so_user_id));
                fwrite($log, $result . "\n");
            } else {
                fwrite($log, $result . "\n");
                $fail_sql = $wpdb->prepare('SELECT * FROM wp_failed_subscription WHERE so_user_id = %d;', $so_user_id);
                $fail_row = $wpdb->get_row($fail_sql);

                if (!$fail_row) {
                    $wpdb->insert('wp_failed_subscription', array('so_user_id' => $so_user_id, 'data' => json_encode($data)));
                }
            }
        }
    }
    
    function sendNewSubscriptionEmail($user) {
        $body = '<html>
            <body>
                    <div style="text-align: center; padding-top: 5vw;">
                            <img src="https://cdn.legendsofaria.com/images/uploads/2018/10/LoA_logo_300px.png" style="width: 50%; height: auto; max-width: 300px;"/>
                    </div>
                    <div style="text-align: left; padding-top: 5vw; padding-left: 2vw; padding-right: 2vw; font-family: arial !important;">
                            <p>Greetings ' . $user->first_name . ',</p>
                            <p>Thanks for purchasing a Premium Subscription, your benefits are now ready to use! When you log in, you’ll get to enjoy:</p>
                            <ul>
                                    <li>An extra housing plot and character slot per server</li>
                                    <li>Tax free living at its finest</li>
                                    <li>Weekly credits to use on the in-game store</li>
                            </ul>
                            <p>Need to make changes to your subscription?</p>
                    </div>
                    <div style="text-align: center; padding-top: 2vw;">
                            <a href="https://legendsofaria.com/dashboard" style="background-color: #cc0000; border: 1px solid black; font-weight: 700; padding: 10px 50px; color: #FFF; text-decoration: none; font-family: arial;">View My Account</a>
                    </div>
                    <div style="text-align: left; padding-top: 2vw; padding-left: 2vw; padding-right: 2vw; font-family: arial !important; padding-bottom: 5vw;">
                            <p>Enjoy!</p>
                            <p>The Citadel Studios Team</p>
                    </div>
            </body>
        </html>';
        $to = $user_info->user_email;
        $subject = 'Your Premium Subscription benefits are ready to use';
        $headers = array('Content-Type: text/html; charset=UTF-8');
        wp_mail($to, $subject, $body, $headers);
    }
    
    function sendCancellationEmail($user) {
        $body = '<html>
            <body>
                    <div style="text-align: center; padding-top: 5vw;">
                            <img src="https://cdn.legendsofaria.com/images/uploads/2018/10/LoA_logo_300px.png" style="width: 50%; height: auto; max-width: 300px;"/>
                    </div>
                    <div style="text-align: left; padding-top: 5vw; padding-left: 2vw; padding-right: 2vw; font-family: arial !important;">
                            <p>Greetings ' . $user->first_name . ',</p>
                            <p>Your Premium Subscription is going to expire and you’ll lose access to your upgraded housing plots, character slots, tax exemptions, and more! Didn’t mean to cancel?</p>
                    </div>
                    <div style="text-align: center; padding-top: 2vw;">
                            <a href="https://legendsofaria.com/dashboard" style="background-color: #cc0000; border: 1px solid black; font-weight: 700; padding: 10px 40px; color: #FFF; text-decoration: none; font-family: arial;">Renew My Subscription</a>
                    </div>
                    <div style="text-align: left; padding-top: 2vw; padding-left: 2vw; padding-right: 2vw; font-family: arial !important; padding-bottom: 5vw;">
                            <p>Cheers,</p>
                            <p>The Citadel Studios Team</p>
                    </div>
            </body>
        </html>';
        $to = $user_info->user_email;
        $subject = 'ALERT: Your Premium Subscription benefits are going to end';
        $headers = array('Content-Type: text/html; charset=UTF-8');
        wp_mail($to, $subject, $body, $headers);
    }
}
?>